# sb::Admin::Template
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Template;
use strict;
use sb::Net::Mailer ();
use sb::Admin::List ();
use base qw( sb::Admin::List );
use vars qw( $VERSION );
$VERSION = '0.21';
# 0.22 [2010/06/13] modified _display_preview to handle link target
# 0.21 [2009/03/19] removed br tags from the strings
# 0.20 [2009/03/18] modofief %mTemplateConfig to add new settings
# 0.19 [2009/02/11] modified callback to get the current template correctly
# 0.18 [2009/02/10] modified _display_details to get editor properly
# 0.17 [2008/12/01] modified _display_info_link
# 0.16 [2008/12/01] modified _display_details to display information correctly
# 0.15 [2008/08/30] modified _update_template_details to handle empty value correctly
# 0.14 [2007/11/05] added title_name
# 0.13 [2007/09/26] enabled $self->conf->save
# 0.12 [2007/09/17] ported from Serene Bach 2.*
# 0.11 [2007/05/04] changed _check_filename to fix a bug
# 0.10 [2007/04/29] added _check_filename
# 0.09 [2006/09/30] changed _additional_dateformat to check linefeed code
# 0.08 [2006/09/26] supported additional date format
# 0.07 [2005/10/19] changed _save_template to call build_css directly
# 0.06 [2005/10/18] changed _save_template to pass TemplateManager object to builder
# 0.05 [2005/10/01] added _convert_template to implement new template spec
# 0.04 [2005/08/12] changed import_template to convert character code correctly
# 0.03 [2005/07/15] added import_template to allow to call externally
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/06/29] changed interface of changing current template
# 0.00 [2005/04/09] generate
# --------------------------------------
# // defines
sub TEMPLATE       (){ 'template.html' };
sub JSFILE_NAME    (){ 'script/listtable.js' }
sub DEFAULT_EDITOR (){ 'sb::Admin::TemplateEditor' };
sub FLAG_IMAGE     (){ 'image/flag.gif' };
sub DENIED_CHECK   (){ '-' };
sub DEFAULT_TYPE   (){ 'main' };
my %mTemplateConfig = (
	'temp' => ['archive_temp','profile_temp','mobile_temp','css_change',],
	'time' => ['entry_date','entry_time','msg_time','dateinlist','archivelist','time_lang','hidenum_archive','hidenum_category',],
	'type' => ['main','config','import','upload',],
);
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	my $editor = $self->get_editor();
	return( $self->static_path . JSFILE_NAME, $editor->javascripts() );
}
sub editor_name
{
	my $self = shift;
	return DEFAULT_EDITOR;
}
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	return sb->string(' &gt; Edit template') if ($self->mode() eq 'edittemp');
	$title = sb->string(' &gt; Design');
	$title .= sb->string(
		  ($self->tid()  ne '')       ? ' &gt; Edit template'
		: ($self->type() eq 'config') ? ' &gt; Settings'
		: ($self->type() eq 'import') ? ' &gt; Import'
		: ($self->type() eq 'upload') ? ' &gt; Upload'
		:                               '');
	return $title;
}
# --------------------------------------
# // public functions - for sub class
sub import_template
{
	my $self = shift;
	my %param = (
		'cont' => undef,
		'time' => undef,
		@_
	);
	$param{'cont'} =~ s/\x0D\x0A|\x0D|\x0A/\n/g; # unify linefeed
	return if (!$param{'cont'});
	my %elem = ();
	my $parser = sb::Net::Mailer->new;
	my $contents = $parser->text_parse($param{'cont'});
	$elem{'name'} = $parser->extract_head($contents->{'head'},'Subject');
	if ($elem{'name'} ne '')
	{
		$elem{'name'} = $self->lang->convert($elem{'name'},$self->lang->charcode);
		$elem{'name'} = sb::Text->new($elem{'name'})->entitized();
	}
	else
	{
		$elem{'name'} = sb->string('[no name]');
	}
	$elem{'files'} = '';
	return if ($contents->{'boundary'} eq '');
	my $dir = $self->conf->value('conf_dir_base') . $self->conf->value('dir_style');
	my $parts = $parser->multipart($contents);
	my %checker = (
		'base.html'  => 'main',
		'style.css'  => 'css',
		'entry.html' => 'entry',
	);
	foreach my $part ( @{$parts} )
	{
		next if ($part->{'name'} eq '');
		if ($checker{$part->{'name'}})
		{ # main, css, or entry
			$elem{$checker{$part->{'name'}}} = $part->{'body'};
			next;
		}
		# attached files
		my $file = $self->_check_filename($dir,$part->{'name'});
		next if ($file eq '');
		open(TMPOUT,">$file");
		binmode(TMPOUT);
		print TMPOUT $part->{'body'};
		close(TMPOUT);
		chmod($self->conf->value('basic_file_attr'),$file);
		$elem{'files'} .= $part->{'name'} . ',' if (-e $file);
	}
	$elem{'info'} = $parts->[0]->{'body'}; # template information
	foreach my $key ('main','css','entry','info')
	{ # convert text for each part
		$elem{$key} =~ s/\x0D\x0A|\x0D|\x0A/\n/g;
		if ($key eq 'info')
		{
			$elem{$key} =~ s/^\.\./\./;
			$elem{$key} =~ s/\n\.\./\n\./g;
		}
		else
		{ # convert path for parts [backward compatibility]
			$elem{$key} =~ s/\.\/__template\//\{site_parts\}/g;
		}
		$elem{$key} = $self->lang->convert($elem{$key},$self->lang->charcode);
	}
	$elem{'use'} = 0;
	$elem{'gen'} = $param{'time'};
	$elem{'mod'} = $param{'time'};
	my $new = sb::Data->add('Template',%elem);
	sb::Data->update($new) if ($new);
	return $new;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	# --- initialize variables
	$self->{'type'} = DEFAULT_TYPE;
	$self->{'tid'}  = undef;
	$self->generate_accessor('type','tid');
	# --- get type
	my $type = $self->cgi->value('__type');
	$type = DEFAULT_TYPE if ( !$type or !grep(/^\Q$type\E$/,@{$mTemplateConfig{'type'}}) );
	$self->type($type);
	# --- get template id (tid)
	my $check = undef;
	if ($self->mode() eq 'edittemp')
	{ # edit current template
		$check = sb::Data->load('Template','cond'=>{'use'=>1},'limit'=>'use');
		die(sb->string('Failed to get the current template.') . "\n") if (!$check);
	}
	elsif ($self->cgi->value('tid') =~ /^\d+$/)
	{ # tid is indicated
		$check = sb::Data->load('Template','id'=>$self->cgi->value('tid'),'limit'=>'id');
	}
	if ($check)
	{
		$self->tid($check->id);
		$self->type(DEFAULT_TYPE);
	}
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->type() eq 'main')
	{ # --- main
		$msg = 
			  ($self->regi() eq 'change') ? $self->_update_template_list()
			: ($self->tid() ne '')        ? $self->_update_template_details()
			:                               sb->string('Unknown error.');
	}
	elsif ($self->type() eq 'config')
	{ # --- config
		$msg = $self->_update_template_config();
	}
	elsif ($self->type() eq 'import')
	{ # --- import
		if ($self->import_template('cont' => $self->cgi->value('temp_package'),'time' => $self->time->data))
		{
			$msg = sb->string('New template has been added.') . "\n";
			$self->type(DEFAULT_TYPE);
		}
		else
		{
			$msg = sb->string('Unknown error.');
		}
	}
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	  ($self->type() eq 'main')   ? $self->_display_template($temp)
	: ($self->type() eq 'config') ? $self->_display_config($temp)
	: ($self->type() eq 'import') ? 1
	: ($self->type() eq 'upload') ? 1
	:                               undef; # it does not normally happen.
	$temp->num(0);
	$temp->tag('sb_selected_mode' => $self->mode());
	$temp->tag('sb_selected_type' => $self->type());
	$temp->tag('sb_submenu_template_' . $self->type() => 'class="current"');
	$temp->block('sb_template_' . $self->type() => 1);
	$temp->block('sb_template_importable'=>1) if ($self->_check_style_dir);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_template_list
{
	my $self = shift;
	if ($self->cgi->value('del') ne '')
	{
		my @sels = split("\0",$self->cgi->value('sel'));
		my @tmps = sb::Data->load('Template','cond'=>{'id'=>\@sels});
		foreach my $tmp (@tmps)
		{
			next if ($tmp->use);
			$tmp->erase;
		}
		sb::Data->update(@tmps) if (@tmps);
		return sprintf(sb->string('Deleted %d item(s).'),$#tmps + 1) . "\n";
	}
	elsif ($self->cgi->value('changecurrent') ne '')
	{
		my $new_id = $self->cgi->value('use');
		my $old = sb::Data->load('Template','cond'=>{'use'=>1});
		return sb->string('Failed to get the current template.') if (!$old);
		if ($new_id ne '' and $new_id ne $old->id)
		{
			my $new = sb::Data->load('Template','id'=>$new_id);
			return sb->string('Failed to get a template.') if (!$new);
			$old->use(0);
			$new->use(1);
			sb::Build->new()->build_css(sb::Template->new($new->css));
			sb::Data->update($old,$new);
			my $msg = '';
			$msg .= sb->string('The current template has been changed.') . "\n";
			$msg .= sb->string('CSS template has been modified.') . "\n";
			$msg .= sb->string('You need to rebuild your weblog to reflect the changes to the previous articles.');
			$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
			return $msg;
		}
	}
}
sub _update_template_details
{
	my $self = shift;
	my $target = sb::Data->load('Template','id'=>$self->tid());
	return sb->string('Failed to get a template.') . ' ' . $self->tid() if (!$target);
	if ($self->regi() eq 'add')
	{ # save as ...
		my $name = sb::Text->new($self->cgi->value('temp_name'))->entitized();
		$name = $target->name if ($name eq '');
		my $new = sb::Data->add('Template',
			'use'   => 0,
			'name'  => $name,
			'gen'   => $self->time->data,
			'mod'   => $self->time->data,
			'main'  => $target->main,
			'css'   => $target->css,
			'entry' => $target->entry,
			'info'  => $target->info,
		);
		sb::Data->update($new) if ($new);
		$self->tid(undef);
		return ($new)
			? sb->string('New template has been added.') . "\n"
			: sb->string('Unknown error.');
	}
	my $msg = '';
	if ($self->regi() eq 'base')
	{ # save base html part
		my $value = $self->cgi->value('template_main');
		$value = '' if ($value =~ /^\s+$/);
		$target->main($value);
		$msg .= sb->string('HTML template has been modified.') . "\n";
		$msg .= sb->string('You need to rebuild your weblog to reflect the changes to the previous articles.');
		$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
	}
	elsif ($self->regi() eq 'css')
	{ # save css part
		my $value = $self->cgi->value('template_css');
		$value = '' if ($value =~ /^\s+$/);
		$target->css($value);
		sb::Build->new()->build_css(sb::Template->new($target->css)) if ($target->use());
		$msg .= sb->string('CSS template has been modified.') . "\n";
	}
	elsif ($self->regi() eq 'entry')
	{ # save entry html part
		my $value = $self->cgi->value('template_entry');
		$value = '' if ($value =~ /^\s+$/);
		$target->entry($value);
		$msg .= sb->string('HTML template has been modified.') . "\n";
		if ($self->conf->value('conf_entry_archive') eq 'Individual')
		{
			$msg .= sb->string('You need to rebuild your weblog to reflect the changes to the previous articles.');
			$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
		}
	}
	$target->mod($self->time->data); # update time stamp as well
	sb::Data->update($target);
	return $msg;
}
sub _update_template_config
{
	my $self = shift;
	foreach my $key (@{$mTemplateConfig{'temp'}})
	{
		$self->conf->value('conf_' . $key => int($self->cgi->value($key)) );
	}
	foreach my $key (@{$mTemplateConfig{'time'}})
	{
		my $value = $self->cgi->value($key);
		$self->conf->value('conf_' . $key => $value);
	}
	$self->conf->save($self->conf->config_file());
	my $msg = '';
	$msg .= sb->string('Settings have been modified.') . "\n";
	$msg .= sb->string('You need to rebuild your weblog to reflect the changes to the previous articles.');
	$msg .= sprintf(sb->string('&#8658; <a href="%s">Rebuild site</a>'),$self->script_path . '?__mode=build') . "\n";
	return $msg;
}
# --------------------------------------
# // private functions - display
sub _display_template
{
	my $self = shift;
	my $temp = shift;
	my @tmps = sb::Data->load('Template',
		'sort'  => 'mod',
		'order' => 1,
		'id'    => ( $self->tid() eq  '' ) ? undef : $self->tid(),
	);
	$self->listmain(
		'template' => $temp,
		'block'    => 'sb_template_list',
		'objects'  => \@tmps,
		'tags'     => {
			'sb_temp_id'   => 'id',
			'sb_temp_name' => 'name',
			'sb_temp_info' => \&_display_info_link,
			'sb_temp_view' => \&_display_preview,
			'sb_temp_gen'  => 'gen',
			'sb_temp_mod'  => 'mod',
			'sb_temp_use'  => \&_display_current,
			'sb_temp_del'  => \&_display_checkbox,
		},
	);
	# tid is set, so we call _display_details here.
	return if ($self->tid() ne '' and $self->_display_details($temp,$tmps[0]));
	# otherwise, we show the selectors
	my $selector = '';
	foreach my $tmp (@tmps)
	{
		$selector .= '<option value="' . $tmp->id . '"';
		$selector .= ' selected="selected"' if ($tmp->use);
		$selector .= '>' . $tmp->name . '</option>' . "\n";
	}
	$temp->num(0);
	$temp->tag('sb_current_templates'=>$selector);
	$temp->block('sb_template_selector'=>1);
	$temp->block('sb_template_select'=>1);
}
sub _display_details
{
	my $self = shift;
	my $temp = shift;
	my $target = shift;
	return if (!$target);
	$temp->num(0);
	$temp->tag('sb_list_selall_status'=>'disabled="disabled"');
	# --- template editors
	if ($self->cgi->value('disp') ne 'info')
	{
		my $editor = $self->get_editor()->new( %$self );
		$temp->tag('sb_temp_id'=>$target->id);
		foreach my $key ('main','css','entry')
		{
			my $value = sb::Text->new($target->$key());
			$temp->tag('sb_template_' . $key =>$editor->display_editor(
				'id' => 'template_' . $key,
				'code' => 'html',
				'content' => $value->entitized(),
			));
		}
		$temp->block('sb_template_edit'=>1);
		return 1;
	}
	# --- template information
	my $text = sb::Text->new($target->info);
	$text->entitized();
	$temp->tag('sb_temp_information'=>$text->format('autolink'));
	$temp->block('sb_template_info'=>1);
	if ($target->files ne '')
	{
		my $dir = $self->conf->value('conf_srv_base') . $self->conf->value('dir_style');
		my @parts = split(',',$target->files);
		my $list = '';
		for (my $i=0;$i<@parts;$i++)
		{
			$list .= sprintf('<li class="%s"><a href="%s">%s</a></li>' . "\n",
				($i % 2) ? 'odd' : 'even',
				$dir . $parts[$i],
				$parts[$i]
			);
		}
		$temp->tag('sb_temp_partslist'=>$list);
		$temp->block('sb_template_parts'=>1);
	}
	return 1;
}
sub _display_config
{
	my $self = shift;
	my $temp = shift;
	$temp->num(0);
	foreach my $key (@{$mTemplateConfig{'temp'}})
	{
		next if ($key eq 'css_change');
		my $value = $self->conf->value('conf_' . $key);
		$self->template_selector(
			'cms' => $temp,
			'tag' => 'sb_' . $key,
			'now' => $value,
		);
		$temp->tag('sb_' . $key . '_id'=>'&amp;tid=' . $value) if ($value > -1);
	}
	$temp->tag('sb_css_change'=>'selected="selected"') if ($self->conf->value('conf_css_change'));
	foreach my $key (@{$mTemplateConfig{'time'}})
	{
		my $value = sb::Text->new($self->conf->value('conf_' . $key))->entitized();
		if ( $key eq 'time_lang' 
		  or $key eq 'hidenum_archive' 
		  or $key eq 'hidenum_category' )
		{
			$temp->tag('sb_' . $key . '_' . $value => 'selected="selected"');
		}
		else
		{
			$temp->tag('sb_' . $key => $value);
		}
	}
	$self->_additional_dateformat($temp);
}
# --------------------------------------
# // private functions - list handlers
sub _display_info_link
{
	my $self = shift;
	my $obj  = shift;
	return '-' if ($obj->info eq '');
	my $path = $self->script_path . '?__mode=template&amp;disp=info&amp;tid=';
	return '<a href="' . $path . $obj->id . '">' . sb->string('[info]') . '</a>';
}
sub _display_preview
{
	my $self = shift;
	my $obj  = shift;
	my $viewpath = $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_sb');
	$viewpath .= '?tid=' . $obj->id;
	if ( $self->user and $self->user->get_option('linktarg') ne '' )
	{
		$viewpath .= '" target="' . $self->user->get_option('linktarg');
	}
	return $viewpath;
}
sub _display_current
{
	my $self = shift;
	my $obj  = shift;
	my $parts = $self->conf->value('srv_temp') . FLAG_IMAGE;
	return ( !$obj->use )
		? DENIED_CHECK
		: '<img src="' . $parts . '" height="20" width="20" alt="in use" />';
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return ( $self->tid() eq '' and !$obj->use )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" />'
		: DENIED_CHECK;
}
# --------------------------------------
# // private functions
sub _check_style_dir
{
	my $self = shift;
	my $dir = $self->conf->value('conf_dir_base') . $self->conf->value('dir_style');
	return (-d $dir and -r $dir and -w $dir)
}
sub _check_filename
{ # for importing template
	my $self = shift;
	my $base = shift;
	my $name = shift;
	my $dir  = '';
	if ($name =~ m!(.+)/(.+)!)
	{ # name includes dir name
		$dir  = $1;
		$name = $2;
	}
	if ($dir ne '' and $dir =~ /\w+/)
	{
		$base .= $dir . '/';
		if (!-e $base)
		{ # not exists, so we should create it.
			eval { umask(0) };
			mkdir($base,sb::Config->get->value('basic_dir_attr'));
		}
	}
	elsif ($dir ne '')
	{ # invalid dir name
		return;
	}
	return if (!-d $base); # make sure dir exists
	return if ($name !~ /[a-zA-Z0-9\-\._]+/); # invalid file name
	return $base . $name; # verified!
}
sub _additional_dateformat
{
	my $self = shift;
	my $cms = shift;
	my $dir = $self->conf->value('conf_dir_base') . $self->conf->value('dir_style');
	my @files = ();
	opendir(TEMPDIR,$dir);
	foreach my $check ( readdir(TEMPDIR) )
	{
		push(@files,$check) if ($check =~ /\.dateform$/);
	}
	closedir(TEMPDIR);
	my $array  = '<script type="text/javascript">' . "\n" . '// <![CDATA[' . "\n";
	my $option = '';
	foreach my $name (@files)
	{
		my $file = $dir . $name;
		$name =~ s/\.dateform$//;
		my $prefix = "Spica.Utils.DateSet['" . $name . "']";
		open(TEMPFILE,"<$file") or next;
		my @lines = <TEMPFILE>;
		my $tmp = join('',@lines);
		$tmp =~ s/\x0D\x0A|\x0D|\x0A/\n/g;
		@lines = split("\n",$tmp);
		my $label = sb::Text->new(shift(@lines))->entitized();
		$array .= $prefix . ' = new Array();' . "\n";
		foreach my $line ( @lines )
		{
			my ($key,$val) = split(/\t/,$line,2);
			next if ($key =~ /\W/);
			$val =~ s/\\/\\\\/g;
			$val =~ s/\'/\\\'/g;
			$val =~ s/\//\\\//g;
			$array .= $prefix . '.' . $key . ' = \'' . $val . '\';' . "\n";
		}
		$option .= '<option value="' . $name . '">' . $label . '</option>';
		close(TEMPFILE);
	}
	$cms->num(0);
	$cms->tag('sb_additional_date_array'=>$array . '// ]]>' . "\n" . '</script>');
	$cms->tag('sb_additional_date_option'=>$option);
}
1;
__END__
