# sb::Net::Mailer
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Net::Mailer;
use strict;
use sb::Language ();
use sb::Net ();
use base qw( sb::Net );
use vars qw( $VERSION );
$VERSION = '0.08';
# 0.08 [2007/08/15] changed error strings
# 0.07 [2007/08/04] changed initialize
# 0.06 [2007/07/28] added missed methods
# 0.05 [2007/07/16] changed return to make them consistent
# 0.04 [2007/07/04] merged Fuco's patch. Thx Fuco!
# 0.03 [2007/02/14] changed multipart to change bracket rule
# 0.02 [2007/08/01] changed sendmail to handle Cc and Bcc
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/09] created
# --------------------------------------
# // defines / static variables
sub ERROR_NOT_SUPPORTED    (){ "indicated type is not supported : " };
sub ERROR_SMTP_ERROR       (){ "Failed to create Net::SMTP\n" };
sub ERROR_NO_DESTINATION   (){ "no destination address\n" };
sub ERROR_DISABLE_SENDMAIL (){ "cannot use sendmail, please check the path for sendmail\n" };
sub ERROR_NOTOPEN_SENDMAIL (){ "cannot open via sendmail\n" };
sub ERROR_NO_ADDRESS       (){ "need to set pop server address\n" };
sub ERROR_NO_ACCOUNT       (){ "need to set pop account\n" };
sub ERROR_NO_PASSWORD      (){ "need to set pop password\n" };
sub ERROR_POP3_ERROR       (){ "Failed to create Net::POP3ss\n" };
sub ERROR_AUTHORIZATION    (){ "Failed to authorize\n" };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'address'  => 'anonymous@example.com',
		'sender'   => {
			'type'    => 'sendmail',
			'address' => '/usr/sbin/sendmail',
		},
		'receiver' => {
			'address'  => undef,
			'acsount'  => undef,
			'password' => undef,
			'is_apop'  => 0,
		},
		'charcode' => 'jis',
		@_
	);
}
# --------------------------------------
# // public functions
sub sendmail
{
	my $self = shift;
	my %param = (
		'To'      => undef,
		'Cc'      => undef,
		'Bcc'     => undef,
		'From'    => $self->{'address'},
		'Subject' => 'no title',
		'Body'    => '',
		@_,
	);
	my $head = $self->head(%param);
	my $body = sb::Language->get->convert($param{'Body'},$self->{'charcode'});
	eval
	{
		die(ERROR_NO_DESTINATION) if (!$param{'To'});
		my $server = $self->{'sender'}{'address'};
		my $from = $param{'From'};
		SWITCH_TYPE: {
			$_ = $self->{'sender'}{'type'};
			/^sendmail/ && do
			{
				die(ERROR_DISABLE_SENDMAIL) if !(-x $server);
				open(SENDMAIL,"| $server -t -i -f '$from'") or die(ERROR_NOTOPEN_SENDMAIL);
				print SENDMAIL $head;
				print SENDMAIL $body;
				close(SENDMAIL);
				last SWITCH_TYPE;
			};
			/^smtp/ && do
			{
				require Net::SMTP;
				my $smtp = Net::SMTP->new($server,'Hello'=>$server);
				$smtp->mail($from);
				$smtp->to($param{'To'});
				$smtp->cc($param{'Cc'}) if ($param{'Cc'});
				$smtp->bcc($param{'Bcc'}) if ($param{'Bcc'});
				$smtp->data();
				$smtp->datasend($head);
				$smtp->datasend($body);
				$smtp->dataend();
				$smtp->quit;
				last SWITCH_TYPE;
			};
			die(ERROR_NOT_SUPPORTED . $_ . "\n");
		}; # end of SWITCHE_TYPE
	}; # end of eval
	return if ($@);
	return 1;
}
sub head
{
	my $self = shift;
	my %param = @_;
	my $header;
	my $lang = sb::Language->get;
	foreach my $key ( keys(%param) )
	{
		next if ($key eq 'Body');
		$header .= $key . ': ' . $lang->convert($param{$key},$self->{'charcode'}) . "\n";
	}
	$header .= 'MIME-Version: 1.0' . "\n";
	$header .= 'Content-type: text/plain; charset=' . $lang->charset_of($self->{'charcode'}) . "\n";
	$header .= 'Content-Transfer-Encoding: 7bit' . "\n";
	$header .= 'X-Mailer: ' . $self->agent_name . "\n";
	$header .= "\n"; # boundary between head and body
	require 'mimeutil.pl';
	return mimeutil::mimeencode($header);
}
sub receive
{
	my $self = shift;
	my %param = (
		'subj'  => undef, # Subject of mail which will be received
		'msgid' => undef, # Received Message-ID (if the recived mail has same ID, it will be ignored)
		'from'  => undef, # Acceptable From address (array ref.)
		'size'  => undef, # Acceptable mail size (if it's set as -1, there is no limitation)
		'del'   => undef, # Flag for deleting the received mail from server (0 : no / 1 : delete)
		@_,
	);
	my @mail; # received mail
	die(ERROR_NO_ADDRESS) if (! defined($self->{'receiver'}{'address'}) or $self->{'receiver'}{'address'} eq '');
	die(ERROR_NO_ACCOUNT) if (! defined($self->{'receiver'}{'account'}) or $self->{'receiver'}{'account'} eq '');
	die(ERROR_NO_PASSWORD) if (! defined($self->{'receiver'}{'password'}) or $self->{'receiver'}{'password'} eq '');
	eval {
		require Net::POP3;
		my $pop3 = Net::POP3->new($self->{'receiver'}{'address'}) or die(ERROR_POP3_ERROR);
		my $check_auth = undef;
		if ($self->{'receiver'}{'is_apop'})
		{
			$check_auth = $pop3->apop($self->{'receiver'}{'account'},$self->{'receiver'}{'password'});
		}
		else
		{
			$pop3->user($self->{'receiver'}{'account'});
			$check_auth = $pop3->pass($self->{'receiver'}{'password'});
		}
		die(ERROR_AUTHORIZATION) if (!$check_auth);
		my $msglist = $pop3->list();
		foreach my $msgid (keys %$msglist)
		{
			my ($tmp,$data);
			next if (${$msglist}{$msgid} > $param{'size'} and $param{'size'} > -1); # exceed the size limit
			$tmp = $pop3->top($msgid);
			$data = join('',@$tmp);
			$data = _linefeed($data);
			$data .= "\n" if ($data !~ /\n$/); # add linefeed end of data
			$tmp = $self->extract_head($data,'Subject');
			next if ($tmp eq '');
			next if ($param{'subj'} ne '' and index($tmp,$param{'subj'}) == -1);
			if ($param{'from'})
			{
				my $frm_flag;
				$tmp = $self->extract_head($data,'From'); # extract From
				foreach (@{$param{'from'}})
				{
					$frm_flag = 1 if (index($tmp,$_) > -1);
					last if ($frm_flag);
				}
				next if (!$frm_flag);
			}
			if ($param{'msgid'} ne '')
			{
				$tmp = $self->extract_head($data,'Message-ID');
				last if ($tmp eq $param{'msgid'}); # already received so we can leave
			}
			$tmp = $pop3->get($msgid); # get mail
			$data = join('',@$tmp) . "\n.\n";
			$data = _linefeed($data);
			push(@mail,$data); # store mail
			$pop3->delete($msgid) if ($param{'del'}); # remove mail from server
		}
		$pop3->quit; # finish
	};
	die($@) if ($@); # error
	return @mail;
	# [References] ============================================
	# Let's create POP3 client with Module (in Japanese)
	# http://x68000.q-e-d.net/~68user/net/module-pop3.html
	# Receiving mail by Perl (in Japanese)
	# http://homepage3.nifty.com/hippo2000/perltips/rcvmail.htm
	# =========================================================
}
sub text_parse
{
	my $self = shift;
	my $data = shift;
	my $out = {
		'head' => '',
		'body' => '',
		'boundary' => '',
	};
	$data =~ s/\.\n$/\n/; # delete the last period
	if ($data =~ /^(.*?)\n\n(.*?)\n$/s)
	{
		$out->{'head'} = $1 . "\n";
		$out->{'body'} = $2;
	}
	if ($out->{'head'} =~ /Content-Type:\s?multipart\/mixed;\n?\s?boundary=(?:[\"]?)([\w\'()+,-.\/:=? ]*[\w\'()+,-.\/:=?])[\"]?/soi)
	{
		$out->{'boundary'} = '--' . $1;
	}
	else
	{
		$out->{'boundary'} = '';
	}
	return($out);
}
sub multipart
{
	my $self = shift;
	my $data = shift; # should be outpout of text_parse
	my @out;
	my $boundary = $data->{'boundary'};
	my @parts = split(/\Q$boundary\E/, $data->{'body'});
	shift @parts; # we don't use the last part
	require 'mimeutil.pl';
	for (my $i=0;$i<@parts;$i++)
	{
		if($parts[$i] =~ /(.*?)\n\n(.*)/s)
		{
			my ($head,$body) = ($1,$2);
			$out[$i]->{'head'} = $head;
			$out[$i]->{'type'} = _part_type($head);
			$out[$i]->{'code'} = _part_encode($head);
			$out[$i]->{'name'} = _part_name($head);
			if ($out[$i]->{'code'} =~ /Base64/i)
			{
				$out[$i]->{'body'} = mimeutil::bodydecode($body,'b64');
				$out[$i]->{'body'} .= mimeutil::bdeflush('b64');
			}
			elsif ($out[$i]->{'code'} =~ /Quoted/i)
			{
				$out[$i]->{'body'}  = mimeutil::bodydecode($body,'qp');
				$out[$i]->{'body'} .= mimeutil::bdeflush('qp');
			}
			else
			{
				$out[$i]->{'body'} = $body;
			}
		}
	}
	return(\@out);
}
sub extract_head
{
	my $self = shift;
	my ($head,$elem) = @_;
	my $out;
	require 'mimeutil.pl';
	if ($head =~ /$elem: (.*?)\n([\w|-]*?): /s)
	{ # not last header
		$out = $1;
	}
	elsif ($head =~ /$elem: (.*?)\n$/s)
	{ # the last header
		$out = $1;
	}
	$out = mimeutil::mimedecode($out, 'EUC'); # we just use EUC code
	$out =~ s/\n\s?//g;
	$out =~ s/\s$//;
	return($out);
}
# --------------------------------------
# // private functions
sub _linefeed
{
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	return $_[0];
}
sub _part_type
{ # extract Content-Type
	my $head = shift;
	return $1 if ($head =~ /Content-Type:\s?([^; \n]+)[;]?/si); # from RFC 2045
	return;
}
sub _part_name
{ # extract file name
	my $head = shift;
	if ($head =~ /Content-Disposition:\s?.*?;\n?\s*(filename\*.*)/soi)
	{ # from RFC 2231
		return _decode_2231_filename($1);
	}
	# from http://www.meadowy.org/~gotoh/mew-fake-cdp.html
	return $1 if ($head =~ /Content-Disposition:\s?.*?;\n?\s*filename=([^\s\"]+)/soi);
	if ($head =~ /Content-Disposition:\s?.*?;\n?\s*filename=[\"]?([^\"]+)[\"]?/soi)
	{
		require 'mimeutil.pl';
		return mimeutil::mimedecode($1);
	}
	if ($head =~ /Content-Type:\s?.*?;\n?\s*name=\"([^\"]+)\"/soi)
	{ # invalid against RFC, but many clients implement like this
		require 'mimeutil.pl';
		return mimeutil::mimedecode($1);
	}
	return $1 if ($head =~ /Content-Type:\s?.*?;\n?\s*name=([^\"\s]+)/soi);
	return;
}
sub _decode_2231_filename
{
	my $head = shift;
	my $name = '';
	$name .= $1 while ($head =~ /filename\*[\d]*[\*]?=[\"]?([^;\"]*)[\"]?/g);
	if ($name and $name =~ /^(^[\w-]+)\'[\w]*\'(.*)/)
	{ # uri decode
		my $enc = $1;
		($name = $2) =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2',$1)/eg;
	}
	return $name;
}
sub _part_encode
{ # extract encoding
	my $head = shift;
	return $1 if ($head =~ /Content-Transfer-Encoding:\s?([^\s]+)/oi); # from RFC 2045
	return;
}
1;
__END__
