# sb::Plugin::Reorder
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2008 SimpleBoxes/SerendipityNZ ==

use strict;

if ( !defined $sb::VERSION
  or $sb::VERSION =~ /^2.(\d\d)[BRD]$/i )
{
	die("This plugin works under Serene Bach 3.* only.\n");
}

# ======================================
package sb::Plugin::Reorder;
use sb::Plugin ();
# --------------------------------------
# // plugin registration
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'utf8',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin',
		'name'    => 'Reorder list',
		'text'    => 'Reordering categories, links, recommendations and users',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => version(),
	},
	'file' => 'reorder.txt',
	'data' => undef,
);
sb::Plugin->register_admin_module(
	'mode'   => 'reorder',
	'level'  => 1,
	'module' => 'sb::Admin::Reorder',
);
sub version
{
	return '0.03';
	# 0.03 [2009/05/16] removed unnecessary <br />
	# 0.02 [2009/02/27] ported for Serene bach 3.*
	# 0.01 [2007/06/25] changed _reorder_list to load data correctly
}
# ======================================
package sb::Admin::Reorder;
use sb::Text ();
use sb::Time ();
use sb::Build ();
use sb::Admin::List ();
use base qw( sb::Admin::List );
# --------------------------------------
# // defines
sub TEMPLATE    (){ 'reorder.html' };
sub LIST_OPTION (){ '<option value="%s">%s</option>' };
sub JS_ARRAY    (){ 'gListOptions[%d] = new Array(%s);' };
my @mOrderableList = ('Category','Link','Amazon','User');
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	return ( $self->regi() ) 
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $msg = '';
	if ($self->cgi->value('target_list') ne '')
	{
		my $list = $self->cgi->value('target_list');
		my @sels = split("\0",$self->cgi->value('change_order'));
		my %objs = sb::Data->load_as_hash($list,'cond'=>{'id'=>\@sels},'detail'=>'on');
		my @news = ();
		my $check = undef;
		for (my $i=0;$i<@sels;$i++)
		{
			my $obj = $objs{$sels[$i]};
			if ($obj)
			{
				$obj->order($#sels - $i);
				push(@news,$obj);
				$check = $obj->main if ($list eq 'Category' and $obj->main ne '');
			}
		}
		if ($list eq 'Category' and $check ne '')
		{
			my $main = sb::Data->load('Category','id'=>$check);
			if ($main)
			{
				$main->sub(join(',',map { $_->id } @news) . ',');
				sb::Data->update($main);
			}
		}
		sb::Data->update(@news) if (@news);
		$self->build_list('category_list',\&_update_indexes) if ($list eq 'Category');
		$self->build_list('link_list') if ($list eq 'Link');
		$self->build_list('user_list') if ($list eq 'User');
		$msg = 'Modified.';
	}
	else
	{
		$msg = 'No target is selected.';
	}
	return $self->_display('message'=>sb->string($msg));
}
sub _update_indexes
{
	my $self = shift;
	my $builder = shift;
	my @cats = sb::Data->load('Category');
	foreach my $cat ( @cats )
	{ # category indexes
		next if (!$cat->idx);
		next if ($cat->dir eq '');
		$builder->build_category_index($cat->id);
	}
}
sub _display
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $temp = sb::Template->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my %cat = sb::Data->load_as_hash('Category');
	my %lnk = sb::Data->load_as_hash('Link');
	my $target = '';
	my $option = '';
	my $num = 1;
	foreach my $list ( @mOrderableList )
	{
		my $string = sb->string($list);
		my @elem = ();
		my @buf  = ();
		my @objs = sb::Data->load($list,'sort'=>'order','order'=>1);
		next if (!@objs);
		foreach my $obj (@objs)
		{
			if ($list eq 'Category' and $obj->main != -1 )
			{
				push(@buf,$obj->main);
				next;
			}
			if ($list eq 'Link' and $obj->type =~ /^\d+$/)
			{
				push(@buf,$obj->type);
				next;
			}
			my $name = ($list eq 'User') ? $obj->real : $obj->name;
			$name =~ s/\'/\\\'/g;
			push(@elem,"'" . $obj->id . ':' . sb::Text->detitize($name) . "'");
		}
		$target .= sprintf(LIST_OPTION,$list,$string);
		$option .= sprintf(JS_ARRAY,$num,join(',',@elem)) . "\n";
		$num++;
		if (@buf)
		{
			my %cnt = ();
			@buf = grep(!$cnt{$_}++, @buf);
			foreach my $check (@buf)
			{
				@elem = ();
				my $list_name = ($list eq 'Category') 
					? $cat{$check}->fullname(\%cat) 
					: $lnk{$check}->name;
				foreach my $obj (@objs)
				{
					next if ($list eq 'Category' and $obj->main ne $check);
					next if ($list eq 'Link' and $obj->type ne $check);
					my $name = ($list eq 'Category') ? $obj->fullname(\%cat) : $obj->name;
					$name =~ s/\'/\\\'/g;
					push(@elem,"'" . $obj->id . ':' . sb::Text->detitize($name) . "'");
				} # end of foreach my $obj (@objs)
				if (@elem)
				{
					$target .= sprintf(LIST_OPTION,$list,$string . ' [' . $list_name . ']');
					$option .= sprintf(JS_ARRAY,$num,join(',',@elem)) . "\n";
					$num++;
				}
			} # end of foreach my $check (@buf)
		} # end of if (@buf)
	} # end of foreach my $list (@mOrderableList)
	$temp->num(0);
	$temp->tag('sb_target_list'=>$target);
	$temp->tag('sb_list_options'=>$option);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output);
}
1;
__END__
